# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/airt-client',
                'doc_host': 'https://airtai.github.io',
                'git_url': 'https://github.com/airtai/airt-client',
                'lib_path': 'airt'},
  'syms': { 'airt.cli.api_key': { 'airt.cli.api_key.create': ('cli_key.html#create', 'airt/cli/api_key.py'),
                                  'airt.cli.api_key.details': ('cli_key.html#details', 'airt/cli/api_key.py'),
                                  'airt.cli.api_key.ls': ('cli_key.html#ls', 'airt/cli/api_key.py'),
                                  'airt.cli.api_key.revoke': ('cli_key.html#revoke', 'airt/cli/api_key.py')},
            'airt.cli.cli': {},
            'airt.cli.db': { 'airt.cli.db.details': ('cli_datablob.html#details', 'airt/cli/db.py'),
                             'airt.cli.db.from_azure_blob_storage': ('cli_datablob.html#from_azure_blob_storage', 'airt/cli/db.py'),
                             'airt.cli.db.from_clickhouse': ('cli_datablob.html#from_clickhouse', 'airt/cli/db.py'),
                             'airt.cli.db.from_local': ('cli_datablob.html#from_local', 'airt/cli/db.py'),
                             'airt.cli.db.from_mysql': ('cli_datablob.html#from_mysql', 'airt/cli/db.py'),
                             'airt.cli.db.from_s3': ('cli_datablob.html#from_s3', 'airt/cli/db.py'),
                             'airt.cli.db.ls': ('cli_datablob.html#ls', 'airt/cli/db.py'),
                             'airt.cli.db.rm': ('cli_datablob.html#rm', 'airt/cli/db.py'),
                             'airt.cli.db.tag': ('cli_datablob.html#tag', 'airt/cli/db.py'),
                             'airt.cli.db.to_datasource': ('cli_datablob.html#to_datasource', 'airt/cli/db.py')},
            'airt.cli.ds': { 'airt.cli.ds.details': ('cli_datasource.html#details', 'airt/cli/ds.py'),
                             'airt.cli.ds.dtypes': ('cli_datasource.html#dtypes', 'airt/cli/ds.py'),
                             'airt.cli.ds.head': ('cli_datasource.html#head', 'airt/cli/ds.py'),
                             'airt.cli.ds.ls': ('cli_datasource.html#ls', 'airt/cli/ds.py'),
                             'airt.cli.ds.rm': ('cli_datasource.html#rm', 'airt/cli/ds.py'),
                             'airt.cli.ds.tag': ('cli_datasource.html#tag', 'airt/cli/ds.py'),
                             'airt.cli.ds.train': ('cli_datasource.html#train', 'airt/cli/ds.py')},
            'airt.cli.helper': { 'airt.cli.helper.authenicate_user': ('cli_helper.html#authenicate_user', 'airt/cli/helper.py'),
                                 'airt.cli.helper.customize_output_format': ( 'cli_helper.html#customize_output_format',
                                                                              'airt/cli/helper.py'),
                                 'airt.cli.helper.display_formated_table': ('cli_helper.html#display_formated_table', 'airt/cli/helper.py'),
                                 'airt.cli.helper.echo_formatted_output': ('cli_helper.html#echo_formatted_output', 'airt/cli/helper.py'),
                                 'airt.cli.helper.get_example_for_type': ('cli_helper.html#get_example_for_type', 'airt/cli/helper.py'),
                                 'airt.cli.helper.get_example_output_format': ( 'cli_helper.html#get_example_output_format',
                                                                                'airt/cli/helper.py'),
                                 'airt.cli.helper.get_phone_registration_status': ( 'cli_helper.html#get_phone_registration_status',
                                                                                    'airt/cli/helper.py'),
                                 'airt.cli.helper.humanize_date': ('cli_helper.html#humanize_date', 'airt/cli/helper.py'),
                                 'airt.cli.helper.humanize_number': ('cli_helper.html#humanize_number', 'airt/cli/helper.py'),
                                 'airt.cli.helper.humanize_size': ('cli_helper.html#humanize_size', 'airt/cli/helper.py'),
                                 'airt.cli.helper.requires_auth_token': ('cli_helper.html#requires_auth_token', 'airt/cli/helper.py'),
                                 'airt.cli.helper.requires_totp': ('cli_helper.html#requires_totp', 'airt/cli/helper.py'),
                                 'airt.cli.helper.requires_totp_or_otp': ('cli_helper.html#requires_totp_or_otp', 'airt/cli/helper.py'),
                                 'airt.cli.helper.separate_integers_and_strings': ( 'cli_helper.html#separate_integers_and_strings',
                                                                                    'airt/cli/helper.py')},
            'airt.cli.model': { 'airt.cli.model.details': ('cli_model.html#details', 'airt/cli/model.py'),
                                'airt.cli.model.evaluate': ('cli_model.html#evaluate', 'airt/cli/model.py'),
                                'airt.cli.model.ls': ('cli_model.html#ls', 'airt/cli/model.py'),
                                'airt.cli.model.predict': ('cli_model.html#predict', 'airt/cli/model.py'),
                                'airt.cli.model.rm': ('cli_model.html#rm', 'airt/cli/model.py')},
            'airt.cli.pred': { 'airt.cli.pred.details': ('cli_pred.html#details', 'airt/cli/pred.py'),
                               'airt.cli.pred.ls': ('cli_pred.html#ls', 'airt/cli/pred.py'),
                               'airt.cli.pred.rm': ('cli_pred.html#rm', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_azure_blob_storage': ('cli_pred.html#to_azure_blob_storage', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_clickhouse': ('cli_pred.html#to_clickhouse', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_local': ('cli_pred.html#to_local', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_mysql': ('cli_pred.html#to_mysql', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_pandas': ('cli_pred.html#to_pandas', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_s3': ('cli_pred.html#to_s3', 'airt/cli/pred.py')},
            'airt.cli.token': {'airt.cli.token.token': ('cli_token.html#token', 'airt/cli/token.py')},
            'airt.cli.user': { 'airt.cli.user.create': ('cli_user.html#create', 'airt/cli/user.py'),
                               'airt.cli.user.details': ('cli_user.html#details', 'airt/cli/user.py'),
                               'airt.cli.user.disable': ('cli_user.html#disable', 'airt/cli/user.py'),
                               'airt.cli.user.enable': ('cli_user.html#enable', 'airt/cli/user.py'),
                               'airt.cli.user.ls': ('cli_user.html#ls', 'airt/cli/user.py'),
                               'airt.cli.user.register_phone_number': ('cli_user.html#register_phone_number', 'airt/cli/user.py'),
                               'airt.cli.user.reset_password': ('cli_user.html#reset_password', 'airt/cli/user.py'),
                               'airt.cli.user.update': ('cli_user.html#update', 'airt/cli/user.py')},
            'airt.cli.version': {'airt.cli.version.version': ('cli_version.html#version', 'airt/cli/version.py')},
            'airt.client': {},
            'airt.components.api_key': { 'airt.components.api_key.APIKey': ('api_keys.html#apikey', 'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.__init__': ( 'api_keys.html#apikey.__init__',
                                                                                      'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey._get_key_names': ( 'api_keys.html#apikey._get_key_names',
                                                                                            'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.as_df': ( 'api_keys.html#apikey.as_df',
                                                                                   'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.create': ( 'api_keys.html#apikey.create',
                                                                                    'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.details': ( 'api_keys.html#apikey.details',
                                                                                     'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.ls': ('api_keys.html#apikey.ls', 'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.revoke': ( 'api_keys.html#apikey.revoke',
                                                                                    'airt/components/api_key.py')},
            'airt.components.client': { 'airt.components.client.Client': ('api_client.html#client', 'airt/components/client.py'),
                                        'airt.components.client.Client.__init__': ( 'api_client.html#client.__init__',
                                                                                    'airt/components/client.py'),
                                        'airt.components.client.Client._delete_data': ( 'api_client.html#client._delete_data',
                                                                                        'airt/components/client.py'),
                                        'airt.components.client.Client._get_data': ( 'api_client.html#client._get_data',
                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client._get_server_url_and_token': ( 'api_client.html#client._get_server_url_and_token',
                                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client._post_data': ( 'api_client.html#client._post_data',
                                                                                      'airt/components/client.py'),
                                        'airt.components.client.Client.get_token': ( 'api_client.html#client.get_token',
                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client.set_sso_token': ( 'api_client.html#client.set_sso_token',
                                                                                         'airt/components/client.py'),
                                        'airt.components.client.Client.set_token': ( 'api_client.html#client.set_token',
                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client.version': ( 'api_client.html#client.version',
                                                                                   'airt/components/client.py'),
                                        'airt.components.client._get_credentials': ( 'api_client.html#_get_credentials',
                                                                                     'airt/components/client.py')},
            'airt.components.datablob': { 'airt.components.datablob.DataBlob': ( 'api_datablob.html#datablob',
                                                                                 'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.__init__': ( 'api_datablob.html#datablob.__init__',
                                                                                          'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._get_cloud_provider_and_region': ( 'api_datablob.html#datablob._get_cloud_provider_and_region',
                                                                                                                'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._get_default_provider_and_regions': ( 'api_datablob.html#datablob._get_default_provider_and_regions',
                                                                                                                   'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._get_tag_name_and_datasource_id': ( 'api_datablob.html#datablob._get_tag_name_and_datasource_id',
                                                                                                                 'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._upload_to_s3_with_retry': ( 'api_datablob.html#datablob._upload_to_s3_with_retry',
                                                                                                          'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.as_df': ( 'api_datablob.html#datablob.as_df',
                                                                                       'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.delete': ( 'api_datablob.html#datablob.delete',
                                                                                        'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.details': ( 'api_datablob.html#datablob.details',
                                                                                         'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_azure_blob_storage': ( 'api_datablob.html#datablob.from_azure_blob_storage',
                                                                                                         'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_clickhouse': ( 'api_datablob.html#datablob.from_clickhouse',
                                                                                                 'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_local': ( 'api_datablob.html#datablob.from_local',
                                                                                            'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_mysql': ( 'api_datablob.html#datablob.from_mysql',
                                                                                            'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_s3': ( 'api_datablob.html#datablob.from_s3',
                                                                                         'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.is_ready': ( 'api_datablob.html#datablob.is_ready',
                                                                                          'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.ls': ( 'api_datablob.html#datablob.ls',
                                                                                    'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.progress_bar': ( 'api_datablob.html#datablob.progress_bar',
                                                                                              'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.set_default_cloud_provider': ( 'api_datablob.html#datablob.set_default_cloud_provider',
                                                                                                            'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.tag': ( 'api_datablob.html#datablob.tag',
                                                                                     'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.to_datasource': ( 'api_datablob.html#datablob.to_datasource',
                                                                                               'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.wait': ( 'api_datablob.html#datablob.wait',
                                                                                      'airt/components/datablob.py'),
                                          'airt.components.datablob._docstring_example': ( 'api_datablob.html#_docstring_example',
                                                                                           'airt/components/datablob.py')},
            'airt.components.datasource': { 'airt.components.datasource.DataSource': ( 'api_datasource.html#datasource',
                                                                                       'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.__init__': ( 'api_datasource.html#datasource.__init__',
                                                                                                'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.as_df': ( 'api_datasource.html#datasource.as_df',
                                                                                             'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.delete': ( 'api_datasource.html#datasource.delete',
                                                                                              'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.details': ( 'api_datasource.html#datasource.details',
                                                                                               'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.dtypes': ( 'api_datasource.html#datasource.dtypes',
                                                                                              'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.head': ( 'api_datasource.html#datasource.head',
                                                                                            'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.is_ready': ( 'api_datasource.html#datasource.is_ready',
                                                                                                'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.ls': ( 'api_datasource.html#datasource.ls',
                                                                                          'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.progress_bar': ( 'api_datasource.html#datasource.progress_bar',
                                                                                                    'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.tag': ( 'api_datasource.html#datasource.tag',
                                                                                           'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.train': ( 'api_datasource.html#datasource.train',
                                                                                             'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.wait': ( 'api_datasource.html#datasource.wait',
                                                                                            'airt/components/datasource.py'),
                                            'airt.components.datasource._docstring_example': ( 'api_datasource.html#_docstring_example',
                                                                                               'airt/components/datasource.py')},
            'airt.components.model': { 'airt.components.model.Model': ('api_model.html#model', 'airt/components/model.py'),
                                       'airt.components.model.Model.__init__': ( 'api_model.html#model.__init__',
                                                                                 'airt/components/model.py'),
                                       'airt.components.model.Model.as_df': ('api_model.html#model.as_df', 'airt/components/model.py'),
                                       'airt.components.model.Model.delete': ('api_model.html#model.delete', 'airt/components/model.py'),
                                       'airt.components.model.Model.details': ('api_model.html#model.details', 'airt/components/model.py'),
                                       'airt.components.model.Model.evaluate': ( 'api_model.html#model.evaluate',
                                                                                 'airt/components/model.py'),
                                       'airt.components.model.Model.ls': ('api_model.html#model.ls', 'airt/components/model.py'),
                                       'airt.components.model.Model.predict': ('api_model.html#model.predict', 'airt/components/model.py'),
                                       'airt.components.model._docstring_example': ( 'api_model.html#_docstring_example',
                                                                                     'airt/components/model.py')},
            'airt.components.prediction': { 'airt.components.prediction.Prediction': ( 'api_prediction.html#prediction',
                                                                                       'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.__init__': ( 'api_prediction.html#prediction.__init__',
                                                                                                'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction._download_prediction_file_to_local': ( 'api_prediction.html#prediction._download_prediction_file_to_local',
                                                                                                                          'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.as_df': ( 'api_prediction.html#prediction.as_df',
                                                                                             'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.delete': ( 'api_prediction.html#prediction.delete',
                                                                                              'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.details': ( 'api_prediction.html#prediction.details',
                                                                                               'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.ls': ( 'api_prediction.html#prediction.ls',
                                                                                          'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_azure_blob_storage': ( 'api_prediction.html#prediction.to_azure_blob_storage',
                                                                                                             'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_clickhouse': ( 'api_prediction.html#prediction.to_clickhouse',
                                                                                                     'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_local': ( 'api_prediction.html#prediction.to_local',
                                                                                                'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_mysql': ( 'api_prediction.html#prediction.to_mysql',
                                                                                                'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_pandas': ( 'api_prediction.html#prediction.to_pandas',
                                                                                                 'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_s3': ( 'api_prediction.html#prediction.to_s3',
                                                                                             'airt/components/prediction.py'),
                                            'airt.components.prediction._docstring_example': ( 'api_prediction.html#_docstring_example',
                                                                                               'airt/components/prediction.py')},
            'airt.components.progress_status': { 'airt.components.progress_status.ProgressStatus': ( 'api_progressstatus.html#progressstatus',
                                                                                                     'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.__init__': ( 'api_progressstatus.html#progressstatus.__init__',
                                                                                                              'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.is_ready': ( 'api_progressstatus.html#progressstatus.is_ready',
                                                                                                              'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.progress_bar': ( 'api_progressstatus.html#progressstatus.progress_bar',
                                                                                                                  'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.wait': ( 'api_progressstatus.html#progressstatus.wait',
                                                                                                          'airt/components/progress_status.py')},
            'airt.components.user': { 'airt.components.user.User': ('api_user.html#user', 'airt/components/user.py'),
                                      'airt.components.user.User.__init__': ('api_user.html#user.__init__', 'airt/components/user.py'),
                                      'airt.components.user.User._get_mfa_provision_url': ( 'api_user.html#user._get_mfa_provision_url',
                                                                                            'airt/components/user.py'),
                                      'airt.components.user.User.activate_mfa': ( 'api_user.html#user.activate_mfa',
                                                                                  'airt/components/user.py'),
                                      'airt.components.user.User.as_df': ('api_user.html#user.as_df', 'airt/components/user.py'),
                                      'airt.components.user.User.create': ('api_user.html#user.create', 'airt/components/user.py'),
                                      'airt.components.user.User.details': ('api_user.html#user.details', 'airt/components/user.py'),
                                      'airt.components.user.User.disable': ('api_user.html#user.disable', 'airt/components/user.py'),
                                      'airt.components.user.User.disable_mfa': ( 'api_user.html#user.disable_mfa',
                                                                                 'airt/components/user.py'),
                                      'airt.components.user.User.disable_sso': ( 'api_user.html#user.disable_sso',
                                                                                 'airt/components/user.py'),
                                      'airt.components.user.User.enable': ('api_user.html#user.enable', 'airt/components/user.py'),
                                      'airt.components.user.User.enable_mfa': ('api_user.html#user.enable_mfa', 'airt/components/user.py'),
                                      'airt.components.user.User.enable_sso': ('api_user.html#user.enable_sso', 'airt/components/user.py'),
                                      'airt.components.user.User.ls': ('api_user.html#user.ls', 'airt/components/user.py'),
                                      'airt.components.user.User.register_phone_number': ( 'api_user.html#user.register_phone_number',
                                                                                           'airt/components/user.py'),
                                      'airt.components.user.User.reset_password': ( 'api_user.html#user.reset_password',
                                                                                    'airt/components/user.py'),
                                      'airt.components.user.User.send_sms_otp': ( 'api_user.html#user.send_sms_otp',
                                                                                  'airt/components/user.py'),
                                      'airt.components.user.User.update': ('api_user.html#user.update', 'airt/components/user.py'),
                                      'airt.components.user.User.validate_phone_number': ( 'api_user.html#user.validate_phone_number',
                                                                                           'airt/components/user.py')},
            'airt.constant': {},
            'airt.docstring.helpers': { 'airt.docstring.helpers._extract_examples_from_docstring': ( 'docstring_helpers.html#_extract_examples_from_docstring',
                                                                                                     'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers._format_output': ( 'docstring_helpers.html#_format_output',
                                                                                   'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers._get_keywords': ( 'docstring_helpers.html#_get_keywords',
                                                                                  'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers._replace_keywords': ( 'docstring_helpers.html#_replace_keywords',
                                                                                      'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers.run_examples_from_docstring': ( 'docstring_helpers.html#run_examples_from_docstring',
                                                                                                'airt/docstring/helpers.py')},
            'airt.helper': { 'airt.helper._get_json': ('api_helper.html#_get_json', 'airt/helper.py'),
                             'airt.helper.add_example_to_docs': ('api_helper.html#add_example_to_docs', 'airt/helper.py'),
                             'airt.helper.add_ready_column': ('api_helper.html#add_ready_column', 'airt/helper.py'),
                             'airt.helper.check_and_append_otp_query_param': ( 'api_helper.html#check_and_append_otp_query_param',
                                                                               'airt/helper.py'),
                             'airt.helper.delete_data': ('api_helper.html#delete_data', 'airt/helper.py'),
                             'airt.helper.dict_to_df': ('api_helper.html#dict_to_df', 'airt/helper.py'),
                             'airt.helper.ensure_is_instance': ('api_helper.html#ensure_is_instance', 'airt/helper.py'),
                             'airt.helper.generate_df': ('api_helper.html#generate_df', 'airt/helper.py'),
                             'airt.helper.get_attributes_from_instances': ( 'api_helper.html#get_attributes_from_instances',
                                                                            'airt/helper.py'),
                             'airt.helper.get_base_url': ('api_helper.html#get_base_url', 'airt/helper.py'),
                             'airt.helper.get_data': ('api_helper.html#get_data', 'airt/helper.py'),
                             'airt.helper.get_values_from_item': ('api_helper.html#get_values_from_item', 'airt/helper.py'),
                             'airt.helper.post_data': ('api_helper.html#post_data', 'airt/helper.py'),
                             'airt.helper.standardize_phone_number': ('api_helper.html#standardize_phone_number', 'airt/helper.py')},
            'airt.logger': { 'airt.logger.CLILoggerHandler': ('logger.html#cliloggerhandler', 'airt/logger.py'),
                             'airt.logger.CLILoggerHandler.emit': ('logger.html#cliloggerhandler.emit', 'airt/logger.py'),
                             'airt.logger.get_logger': ('logger.html#get_logger', 'airt/logger.py'),
                             'airt.logger.set_level': ('logger.html#set_level', 'airt/logger.py'),
                             'airt.logger.supress_timestamps': ('logger.html#supress_timestamps', 'airt/logger.py')},
            'airt.sanitizer': { 'airt.sanitizer.Logger._log': ('sanitize_secrets.html#logger._log', 'airt/sanitizer.py'),
                                'airt.sanitizer.new_publish_display_data': ( 'sanitize_secrets.html#new_publish_display_data',
                                                                             'airt/sanitizer.py'),
                                'airt.sanitizer.sanitize_secrets': ('sanitize_secrets.html#sanitize_secrets', 'airt/sanitizer.py'),
                                'airt.sanitizer.sanitized_print': ('sanitize_secrets.html#sanitized_print', 'airt/sanitizer.py')},
            'airt.testing.activate_by_import': { 'airt.testing.activate_by_import.activated': ( 'activatetestingenvironment.html#activated',
                                                                                                'airt/testing/activate_by_import.py')}}}
