# AUTOGENERATED! DO NOT EDIT! File to edit: ../notebooks/Constants.ipynb.

# %% auto 0
__all__ = ['CLIENT_DB_USERNAME', 'CLIENT_DB_PASSWORD', 'SERVICE_USERNAME', 'SERVICE_PASSWORD', 'SERVER_URL', 'SERVICE_TOKEN',
           'SERVICE_SUPER_USER', 'PROD_URL', 'CLIENT_NAME']

# %% ../notebooks/Constants.ipynb 2
# The below are the names of the envirnment variables and not actual values
CLIENT_DB_USERNAME = "AIRT_CLIENT_DB_USERNAME"
CLIENT_DB_PASSWORD = "AIRT_CLIENT_DB_PASSWORD"  # nosec B105
SERVICE_USERNAME = "AIRT_SERVICE_USERNAME"
SERVICE_PASSWORD = "AIRT_SERVICE_PASSWORD"  # nosec B105
SERVER_URL = "AIRT_SERVER_URL"
SERVICE_TOKEN = "AIRT_SERVICE_TOKEN"  # nosec B105
SERVICE_SUPER_USER = "AIRT_SERVICE_SUPER_USER"
PROD_URL = "https://api.airt.ai"
CLIENT_NAME = "airt"
