# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/ActivateTestingEnvironment.ipynb.

# %% auto 0
__all__ = ['activated']

# %% ../../notebooks/ActivateTestingEnvironment.ipynb 2
def activated() -> bool:
    return True

# %% ../../notebooks/ActivateTestingEnvironment.ipynb 3
from airt.logger import get_logger, set_level, supress_timestamps  # type: ignore

supress_timestamps()

# %% ../../notebooks/ActivateTestingEnvironment.ipynb 4
import logging

# %% ../../notebooks/ActivateTestingEnvironment.ipynb 5
logger = get_logger(__name__)

set_level(logging.INFO)

logger.info("Testing environment activated.")

# %% ../../notebooks/ActivateTestingEnvironment.ipynb 6
import typer

typer.core.rich = None  # type: ignore
